const models = require(process.cwd());
const LoansFactory = require('./factories/Loans');
const PropertyDetailsFactory = require('./factories/PropertyDetails');
const LoanStatusesFactory = require('./factories/LoanStatuses');
const CommentsFactory = require('./factories/Comments');
const BorrowersFactory = require('./factories/Borrowers');
const DocumentRequestsFactory = require('./factories/DocumentRequests');

test('should have the correct schema', async () => {
  const loan = await LoansFactory();
  expect(LoansFactory.validate(loan.toJSON())).toBeTruthy();
});

test('should have a correct schema if PropertyDetails is included', async () => {
  const propertyDetail = await PropertyDetailsFactory();
  const { id } = await LoansFactory({ propertyDetailId: propertyDetail.id });

  const loan = await models.Loans.findOne({
    where: { id },
    include: [
      { model: models.PropertyDetails },
    ],
  });

  expect(PropertyDetailsFactory.validate(loan.propertyDetail.toJSON())).toBeTruthy();
  expect(LoansFactory.validate(loan.toJSON())).toBeTruthy();
});

// eager loading will not work, need to verify use case in the micro services and/or webapp
test.skip('should have a correct schema if LoanStatuses is included', async () => {
  const loanStatuses = await Promise.all([LoanStatusesFactory(), LoanStatusesFactory()]);

  const { id } = await LoansFactory();

  const loan = await models.Loans.findOne({
    where: { id },
    include: [
      /*
        There is no direct db link between LoanStatuses and Loans
        but this is the use case found in
        https://bitbucket.org/pluminc/plum.microservices.loan-service/src/9c0df862ccef55e1679bfade42cbbbf6483eadcb/handlers/fetchLoans.js?at=develop#fetchLoans.js-23,30
        https://bitbucket.org/pluminc/plum.microservices.loan-service/src/9c0df862ccef55e1679bfade42cbbbf6483eadcb/handlers/fetchLoan.js?at=develop#fetchLoan.js-28
        https://bitbucket.org/pluminc/plum.documentservice/src/b51e0146f6dbb09c2866e52c1351f74e30d81698/loan-service/handler.js?at=develop#handler.js-149,170
        https://bitbucket.org/pluminc/plum.documentservice/src/b51e0146f6dbb09c2866e52c1351f74e30d81698/document-service/handlers/loanHandler.js?at=develop#loanHandler.js-149,170
      */
      {
        model: models.LoanStatuses,
        attributes: ['display'],
        required: false,
      },
    ],
  });

  expect(LoansFactory.validate(loan.toJSON())).toBeTruthy();
});

test('should have a correct schema if Borrowers is included', async () => {
  const { id: loanId } = await LoansFactory();
  const { id: borrowersId } = await BorrowersFactory({ loanId });

  const loan = await models.Loans.findOne({
    where: { id: loanId },
    include: [
      { model: models.Borrowers },
    ],
  });

  expect(LoansFactory.validate(loan.toJSON())).toBeTruthy();

  const { borrowers } = loan;
  expect(borrowers).toHaveLength(1);
  expect(BorrowersFactory.validate(borrowers[0].toJSON())).toBeTruthy();
});

test('should have a correct schema if Comments is included', async () => {
  const { id } = await LoansFactory();
  const comment = await CommentsFactory({ loanId: id });
  
  expect(comment.loanId).toEqual(id);

  const loan = await models.Loans.findAll({
    where: { id },
    include: [
      { model: models.Comments },
    ],
  });

  expect(loan[0].id).toEqual(id);
});

// eager loading will not work, need to verify use case in the micro services and/or webapp
test.skip('should have a correct schema if LoanOriginators is included', async () => {
  const loanOriginators = await Promise.all([
    LoanOriginatorsFactory(),
    LoanOriginatorsFactory(),
  ]);

  const { id } = await LoansFactory();

  const loan = await models.Loans.findOne({
    where: { id },
    include: [
      /*
        There is no db link between LoanOriginators and Loans
        but this is the use case found in
        https://bitbucket.org/pluminc/plum.microservices.uploadeddocument-service/src/32d8da15be3dc418320bb932dcca8cc66bf9bad5/handlers/sendEmailUploadNotification.js?at=develop#sendEmailUploadNotification.js-63
      */
      { 
        model: models.LoanOriginators,
        attributes: ['id', 'firstname', 'lastname', 'primaryEmail'],
      },
    ],
  });
  expect(LoansFactory.validate(loan.toJSON())).toBeTruthy();
  expect(loan.loanOriginators).toHaveLength(2);
});

test('should have a correct schema if DocumentRequests is included', async () => {
  const { id } = await LoansFactory();
  const docReq = await DocumentRequestsFactory({ loanId: id });
  
  expect(docReq.loanId).toEqual(id);

  const loan = await models.Loans.findOne({
    where: { id },
    include: [
      { model: models.DocumentRequests },
    ],
  });

  expect(loan.id).toEqual(id);

  const [docReqPopulate] = loan.documentRequests;
  expect(DocumentRequestsFactory.validate(docReqPopulate.toJSON())).toBeTruthy();
});